<?php
/*
USAGE EXAMPLES 
--------------

//Accept -f --file , and is required
command_option::add("file", Array('f', 'file', 'Please provide the file'), true)

//Accept -d --dir , and is optional
command_option::add("d", Array('d', 'dir', 'Move the backup to option dir'))

//Accept -s  
command_option::add_option("system_mode", array('s', 'Please set this if you want system mode'))

$options = command_options::get_opts(); 
// U should have $options['file'], $options['d'], and $options['system_mode'] available to you
*/

/*
Author: Dawa Lama Sherpa
Contact: dawalama at gmail dot com
NOTE: command_option class provides an interface to deal with command line options 
Requires : PHP5
*/

/* CODE */

Class command_option_exception extends Exception
{
    public function __construct( $message, $code = 0 ) {
        $this->message = $message;  
        parent::__construct( $message, $code ); 
    }

    function __toString( ) {
        return __CLASS__." : [{$this->code}]: {$this->message}\n"; 
    }
}

/*
   Store command line arguments in this object
 */
Class arguments 
{
    private $argv; 
    private $counter; 
    private $filename; 

    function arguments( )
    {
        $this->argv = $_SERVER['argv']; 
        $this->filename = array_shift( $this->argv ); 
        $this->counter = -1;
    }

    function filename()
    {
        return $this->filename;  
    }

    function next( )
    {
        if (isset( $this->argv[$this->counter+1])) {
            $this->counter++; 
            return $this->argv[$this->counter]; 
        }
    }


    function undo_next() 
    {
        $this->counter--;
        if ( $this->counter < 0 ) {
            $this->counter = 0;   
        }
    }
}

Class command_option
{
    static public $_options;
    static public $args;

    static function add_option( $target, $opt )
    {
        self::add( $target, Array($opt[0], false, $opt[1]), false, 0,  true ); 
    }

    static function add( $target, $opt, $required = false, $default=null, $option=false)
    {
        global $_options; 
        if (isset( $_optoins['options'][$target] )) {
            throw new command_option_exception("Target exists"); 
        }else{
            $short_option = array_shift( $opt ); 
            $long_option = array_shift( $opt ); 
            $description = array_shift( $opt ); 
            if ( $required === true ) {
                $_options['required'][] = $target; 
            }
            if ( $option ) {
                $_options['_option'][] = $short_option; 
            }

            $_options['targets'][$target] = $default;  
            $_options['short_options'][$short_option] = $target;
            if ( $long_option === false ){
                $_options['long_options'][$short_option] = $target;
            }else{
                $_options['long_options'][$long_option] = $target;
            }
            $_options['description'][$target] = $description;
        }
    }

    static function get_opts()
    {
        global $_options, $args;
        $args = new arguments( );
        try {
            while ( $arg = $args->next() )
            {
                $opt = self::is_option($arg); 
                if (is_array( $opt )) {
                    if ($opt[0] == 1 ){
                        $short_option = $opt[1];

                        if ( isset($_options['_option']) && in_array($short_option , $_options['_option']) ){
                            $_options['targets'][$_options['short_options'][$short_option]] = 1;  
                        }else{

                            if (in_array( $short_option, array_keys($_options['short_options']) )) {
                                $target_name =$_options['short_options'][$short_option];
                                $next_arg = $args->next(); 
                                self::set_target( $target_name, $next_arg ); 
                            }else{
                                throw new command_option_exception( "Unknown option \"$short_option\""); 
                            }
                        }
                    }elseif ( $opt[0] == 2 ){
                        $long_option = $opt[1];
                        if (in_array( $long_option, array_keys($_options['long_options']) )) {
                            $target_name = $_options['long_options'][$long_option];
                            $next_arg = $args->next(); 
                            self::set_target( $target_name, $next_arg ); 
                        }else{
                            throw new command_option_exception( "Unknown option \"$long_option\""); 
                        }
                    }else{
                        //Extra parameters;
                        $_options['_extra_params'][] = $arg; 
                    }
                }
            }
            if (count($_options['required']) > 0 ){
                $failed_on_require = Array(); 
                foreach ( $_options['required'] as $req ) {
                    if ( ($_options['targets'][$req] === false) || ($_options['targets'][$req] == '' )){
                        $failed_on_require[] = $req; 
                    }
                }

                if ( count($failed_on_require) > 0 ) {
                    $message =  "Required options not met for variables :(" . implode(",", $failed_on_require) . ")"; 
                    throw new command_option_exception( $message , 3 ); 
                }
                }
            return $_options['targets'];

        } catch ( command_option_exception $e ) {
            echo $e->getMessage(), "\n";
            self::print_usage(); 
        }
    }

    static function print_usage(){
        global $_options, $args; 
        echo "\nusage: ".$args->filename()." [options]"; 
        echo "\nOptions and arguments : "; 
        $short_options = array_flip( $_options['short_options'] ); 
        $long_options = array_flip( $_options['long_options'] ); 
        foreach ( $_options['description'] as $target=>$description ) {
            echo "\n";
            if ( $short_options[$target] == $long_options[$target] ){
                echo "-$short_options[$target]\t"; 
            }else{
                echo "-$short_options[$target] | --$long_options[$target]"; 
            }
            echo "\t:\t"; 
            echo "$description"; 
        }
        echo "\n\n"; 
    }

    static function set_target( $target_name, $next_arg )
    {
        global $_options, $args; 
        if (self::is_option( $next_arg ) !== false) { 
            $_options['targets'][$target_name] = ''; 
            $args->undo_next();  
        }else{
            $_options['targets'][$target_name] = $next_arg;     
        }
    }

    static function is_option( $var ) {
        if (preg_match("/^-([a-zA-Z0-9])/", $var, $match)){
            $option_name = $match[1]; 
            $option_type = 1; 
        }elseif ( preg_match("/^--([a-zA-Z0-9]+)/", $var, $match) ){
            $option_name = $match[1]; 
            $option_type = 2; 
        }else{
            return false; 
        }
        return Array( $option_type, $option_name ); 
    }

}


?>
